import os
{% if "PostgreSQLApp" in app_type %}

POSTGRESQL = {'database': "",
              'user': "",
              'port': 5432
              }
{% endif %}

{% if "RedisApp" in app_type %}

REDIS = {
    "host":'localhost',
    "port": 6379,
    "db": 0
}

{% endif %}

DEBUG = True
TEMPLATE_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "templates")
STATIC_DIR = "statics"
STATIC_PATH = "/static"



import os
import shutil
import subprocess
import unittest

from werkzeug.test import Client
from werkzeug.wrappers import BaseResponse

app = """from testapp import settings
from testapp import views
from testapp.urls import url_map
from flast.app import BaseApp, create_app


class App(BaseApp):
    def __init__(self, settings):
        self.url_map = url_map
        self.views = views
        super(App, self).__init__(settings)

if __name__ == '__main__':
    from werkzeug.serving import run_simple
    app = create_app(App)
    run_simple('0.0.0.0', 5000, app, use_debugger=True, use_reloader=True)

app = create_app(App, settings,  with_static=settings.DEBUG)"""

views = """# create you views here. example:
# from flast.views import View
#
#
# class Index(View):
#     def get(self, request):
#         return self.render_template('index.html')"""

urls = """from werkzeug.routing import Map, Rule

url_map = Map([
         # Rule('/', endpoint='index'),
])"""


class TestFlastInit(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        subprocess.call(["flast-init.py", "testapp"])

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree("testapp")

    def test_directories(self):
        for elem in os.listdir('testapp'):
            self.assertTrue(
                elem in ['statics', 'templates', 'sql', 'views.py', 'app.py',
                         '__init__.py', 'urls.py', 'settings.py'])

    def test_template_files(self):
        with open('testapp/app.py') as fd:
            self.assertEqual(app, fd.read())

        with open('testapp/views.py') as fd:
            self.assertEqual(views, fd.read())

        with open('testapp/urls.py') as fd:
            self.assertEqual(urls, fd.read())

    def test_views(self):
        from testapp.app import app
        c = Client(app, BaseResponse)
        resp = c.get('/')
        # no views/urls exists yet but service is running and responding
        self.assertEqual(resp.status_code, 404)

        from werkzeug.routing import Map, Rule
        url_map = Map([Rule('/', endpoint='index'), ])
        from flast.views import View

        class Index(View):
            def get(self, request):
                return self.render_template('index.html')

        class views(object):
            def __init__(self):
                self.Index = Index

        views = views()
        app.url_map = url_map
        app.views = views
        with open("testapp/templates/index.html", "w") as index:
            index.write("Hello World")

        c = Client(app, BaseResponse)
        resp = c.get('/')
        self.assertEqual(resp.status_code, 200)


if __name__ == '__main__' and __package__ is None:
    unittest.main()



import os
from abc import ABCMeta, abstractproperty

from flast.conf import settings
from jinja2 import Environment, FileSystemLoader
from werkzeug.utils import redirect
from werkzeug.wrappers import Response


class View(object):
    @property
    def authorized_method(self):
        authorized_methods = ["GET", "POST", "PUT", "DELETE", "PATCH"]

        return [method for method in authorized_methods
                if hasattr(self, method.lower())]

    def as_view(self, request, *args, **kwargs):
        template_path = os.path.join(
            os.path.dirname("."), settings.TEMPLATE_DIR)
        self.jinja_env = Environment(
            loader=FileSystemLoader(template_path), autoescape=True)
        return self.dispatch(request, *args, **kwargs)

    def render_template(self, template_name, **context):
        t = self.jinja_env.get_template(template_name)
        return Response(t.render(context), mimetype='text/html')

    def dispatch(self, request, *args, **kwargs):
        if request.method in self.authorized_method:
            return getattr(self, request.method.lower())(request, *args,
                                                         **kwargs)


class CreateView(View):
    """
    This class implement the common create worflow.
    - GET: show the form
    - POST:
      - validate the form
      - save something somewhere
      - redirect to an url
      - or re-render the form with errors
    """
    __metaclass__ = ABCMeta

    @abstractproperty
    def form(self):
        pass

    @abstractproperty
    def template(self):
        pass

    @abstractproperty
    def redirect(self):
        pass

    def get(self, request):
        return self.render_template(self.template)

    def post(self, request):
        form_class = self.form
        form = form_class(request.form)
        if form.validate():
            form.save()
            return redirect(self.redirect)
        else:
            return self.render_template(self.template, errors=form.errors)



import os

from flast.conf import settings
from werkzeug.exceptions import HTTPException
from werkzeug.wrappers import Request
from werkzeug.wsgi import SharedDataMiddleware

try:
    import psycopg2
    from psycopg2 import connect, extras
    PG_ENABLE = True
except ImportError:
    PG_ENABLE = False

try:
    import redis
    REDIS_ENABLE = True
except ImportError:
    REDIS_ENABLE = False


class BaseApp(object):
    def __init__(self):
        super(BaseApp, self).__init__()

    def dispatch_request(self, request):
        adapter = self.url_map.bind_to_environ(request.environ)
        try:
            endpoint, values = adapter.match()
            return getattr(self.views, endpoint.capitalize())().as_view(
                request, **values)
        except HTTPException as e:
            return e

    def wsgi_app(self, environ, start_response):
        request = Request(environ)

        if hasattr(self, "session"):
            sid = request.cookies.get('session')

            if sid is None:
                request.session = self.session.new()
            else:
                request.session = self.session.get(sid)

        response = self.dispatch_request(request)

        if hasattr(self, "session") and request.session.should_save:
            self.session.save(request.session)
            response.set_cookie('session', request.session.sid)
        return response(environ, start_response)

    def __call__(self, environ, start_response):
        return self.wsgi_app(environ, start_response)


class PostgreSQLApp(object):
    def __init__(self):
        if not PG_ENABLE:
            raise ImportError("please install the psycopg2 package in"
                              " order to use postgresql backend")
        self.postgresql = connect(**settings.POSTGRESQL)
        self.sql_path = os.path.join(os.path.dirname("."), 'sql')
        super(PostgreSQLApp, self).__init__()

    def write_pg(self, path, params=None):
        fd = os.path.join(self.sql_path, path)
        with open(fd) as fd:
            try:
                with self.postgresql.cursor(
                        cursor_factory=extras.DictCursor) as cur:
                    cur.execute(fd.read(), params)
                self.postgresql.commit()
            except psycopg2.Error:
                self.postgresql.rollback()
                raise

    def request_pg(self, path, params=None):
        fd = os.path.join(self.sql_path, path)
        with open(fd) as fd:
            with self.postgresql.cursor(
                    cursor_factory=extras.DictCursor) as cur:
                cur.execute(fd.read(), params)
                return cur.fetchall()


class RedisApp(object):
    def __init__(self):
        if not REDIS_ENABLE:
            raise ImportError("please install the redis package"
                              " in order to use redis backend")
        self.redis = redis.Redis(**settings.REDIS)
        super(RedisApp, self).__init__()


def create_app(app, with_static=True):

    app = app()
    if with_static:
        app.wsgi_app = SharedDataMiddleware(app.wsgi_app, {
            settings.STATIC_PATH: os.path.join(
                os.path.dirname('.'), settings.STATIC_DIR)
        })
    return app



from flast.conf import settings
from flast.app import {{ app_type|join(', ') }}, create_app

from {{project}}.urls import url_map
from {{project}} import views


class App({{ app_type|join(', ') }}):
    def __init__(self):
        self.url_map = url_map
        self.views = views
        super(App, self).__init__()

if __name__ == '__main__':
    from werkzeug.serving import run_simple
    app = create_app(App)
    run_simple('0.0.0.0', 5000, app, use_debugger=True, use_reloader=True)

app = create_app(App,with_static=settings.DEBUG)



class Form(object):

    fields = []

    def __init__(self, payload, files=None):
        self.payload = payload
        self.files = files

    def validate(self):
        """
        Fill the form with errrors if any.
        Return True if the form is valide, Falses otherwise
        """

        self.errors = {}
        for field in self.fields:
            errors = getattr(self, field).validate(field, self.payload)
            if errors:
                self.errors[field] = errors
        return not self.errors

    def save(self):
        """
        once the form fields and the form as been validated globally,
        you can do whatever you need with the data
        (like save it in a database or else)
        """
        pass


class Field(object):

    validators = []

    def validate(self, field, payload):
        """
        For each field validator run the validation
        with form data return the list of error
        """

        data = payload[field]
        errors = []
        for validator in self.validators:
            error = validator(data)
            if error:
                errors.append(error)
        return errors


def validate_integer(data):
    try:
        int(data)
    except ValueError:
        return "value must be an integer"



#!/usr/bin/env python

import argparse
import os

from jinja2 import Template

from flast import templates

parser = argparse.ArgumentParser()
parser.add_argument('project', help='Name of the project to create')
parser.add_argument("--pg",
                    required=False,
                    dest="pg",
                    action='store_true',
                    help="create a project with postgresql support")
parser.add_argument("--redis",
                    required=False,
                    dest="rs",
                    action='store_true',
                    help="create a project with redis support")
templates_path = templates.__path__[0]

args = parser.parse_args()

os.mkdir(args.project)
cur_dir = os.path.abspath(args.project)

for elem in os.listdir(templates_path):
    if not elem.endswith("py"):
        continue
    path = os.path.join(templates_path, elem)
    target = os.path.join(cur_dir, elem)

    if not os.path.exists(target):
        template = Template(open(path).read())
        app_type = ["BaseApp"]
        if args.pg:
            app_type.append("PostgreSQLApp")
        if args.rs:
            app_type.append("RedisApp")
        app_type.reverse()
        render = template.render(app_type=app_type, project=args.project)
        with open(target, "w") as fd:
            fd.write(render)

for elem in ["sql", "templates", "statics"]:
    target = os.path.join(cur_dir, elem)
    if not os.path.exists(target):
        os.mkdir(target)



import importlib
import os


def get_settings():
    settings = os.environ.get("FLAST_SETTINGS")
    if settings is None:
        raise ValueError("you need to set the settings environment variable")
    else:
        return importlib.import_module(settings)


settings = get_settings()
